/*
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */
package cz.xelfi.quoridor.emailer

import java.net.URL
import java.util.ResourceBundle
import java.text.MessageFormat
import scala.collection.Map
import scala.collection.mutable.HashMap

abstract class Notify {
  protected def allPlayers(): Map[String,List[Info]]
  protected def emails(ids : Collection[String]): Map[String,String]
  protected def sendEmail(address : String, subject : String, text : String) : Unit
  protected def resignGame(id : Info): Unit

  case class Info(lastMove : Long, gameId : String, toMove : String) {
  }

  def process() {
    val ret = allPlayers()
    val name2Address = emails(ret.keySet)

    for (val player <- ret.keySet) {
        val address = name2Address.getOrElse(player, null)

        val rb = ResourceBundle.getBundle("cz.xelfi.quoridor.emailer.Bundle")
        val sb = new StringBuilder
        sb.append(rb.getString("MSG_HEADER"))
        var subject = rb.getString("MSG_SUBJECT")
        val mf = new MessageFormat(rb.getString("MSG_GAME"))
        val mfCancel = new MessageFormat(rb.getString("MSG_GAME_CANCEL"))
        for (val info <- ret(player)) {
          val delay : Long = (System.currentTimeMillis - info.lastMove) / (3600 * 1000 * 24)
          val arr = new Array[Object](2);
          arr(0) = "http://quoridor.xelfi.cz/games/" + info.gameId
          arr(1) = java.lang.Long.valueOf(delay)
          if (delay > 21) {
            subject = rb.getString("MSG_SUBJECT_CANCEL")
            sb.append(mfCancel.format(arr))
            resignGame(info)
          } else if (delay > 14) {
            subject = rb.getString("MSG_SUBJECT_WARN")
            sb.append(mf.format(arr))
          }
      }
        if (address != null) {
          sendEmail(
              address,
              subject,
              sb.toString
          )
        }
    }
  }
}
