/*
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */

package cz.xelfi.quoridor.freemarkerdor;

import java.io.IOException;
import java.util.Date;
import java.util.Iterator;
import java.util.Properties;
import java.util.ServiceLoader;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import org.openide.util.Exceptions;

/** Send emails.
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
public abstract class EmailService {
    private static EmailService DEFAULT;
    
    public abstract void sendEmail(
        String address,
        String subject,
        String text
    ) throws IOException;

    public static synchronized EmailService getDefault() {
        if (DEFAULT == null) {
            Iterator<EmailService> it = ServiceLoader.load(EmailService.class).iterator();
            DEFAULT = it.hasNext() ? it.next() : new Impl();
        }
        return DEFAULT;
    }

    private static class Impl extends EmailService {
        private final Session session;
        public Impl() {
            Properties props = System.getProperties();
            final String host = "mail.smtp.host";
            if (props.get(host) == null) {
                props.put(host,"localhost");
            }
            session = Session.getDefaultInstance(props, null);
        }

        @Override
        public void sendEmail(String address, String subject, String text) throws IOException {
            try {
                MimeMessage message = new MimeMessage(session);
                message.setFrom(new InternetAddress("quoridor@xelfi.cz"));
                message.setSentDate(new Date());
                message.addRecipient(Message.RecipientType.TO, new InternetAddress(address));
                message.setSubject(subject);
                message.setText(text);
                Transport.send(message);
            } catch (MessagingException ex) {
                throw new IOException(ex);
            }

        }
    }
}
