/*
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */
package cz.xelfi.quoridor.freemarkerdor;

import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.api.client.WebResource.Builder;
import java.net.URI;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Cookie;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

/** Keeps "requests". A request is UUID that maps to a URL which can
 * be executed later.
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
public final class Requests extends Object {
    private final Map<UUID,Builder> requests = new HashMap<UUID, Builder>();
    private final WebResource root;

    Requests(WebResource path) {
        root = path;
    }

    @GET
    @Path("{id}")
    @Produces(MediaType.TEXT_HTML)
    public Response redirect(@PathParam("id") String uuid) {
        UUID r = UUID.fromString(uuid);
        Builder web = requests.get(r);
        if (web == null) {
            return Response.status(Response.Status.NOT_FOUND).build();
        }
        return Response.ok(web.get(String.class)).build();
    }

    boolean isVerified(String uuid) {
        return uuid != null && requests.containsKey(UUID.fromString(uuid));
    }

    URI register(String login, WebResource request) {
        UUID uuid = UUID.randomUUID();
        WebResource callback = root.path(uuid.toString());
        Builder builder = request.queryParam("verified", uuid.toString()).
            cookie(Cookie.valueOf("login=" + login));
        requests.put(uuid, builder);
        return callback.getURI();
    }

}
