/*
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */

package cz.xelfi.quoridor.webidor;

import com.sun.jersey.test.framework.WebAppDescriptor;
import com.sun.jersey.test.framework.AppDescriptor;
import java.util.Properties;
import com.sun.jersey.api.client.GenericType;
import com.sun.jersey.api.client.UniformInterfaceException;
import com.sun.jersey.test.framework.JerseyTest;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;
import javax.ws.rs.core.MediaType;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
public class FinishedGameTest extends JerseyTest {
    static {
        System.setProperty("JERSEY_HTTP_PORT", "33435");
    }
    private File dir;
    
    @Override
    protected AppDescriptor configure() {
        try {
            dir = File.createTempFile("quoridor", ".dir");
            dir.delete();
            System.setProperty("quoridor.dir", dir.getPath());
            dir.mkdirs();
            File passwd = new File(dir, "passwd");
            FileOutputStream os = new FileOutputStream(passwd);
            os.write("Jarda=heslo\nJirka=pesko\nMaster=mr\n".getBytes("UTF-8"));
            os.close();
        } catch (Exception ex) {
            throw new IllegalStateException(ex);
        }
        return new WebAppDescriptor.Builder("cz.xelfi.quoridor.webidor.resources").build();
    }

    @Override
    public void tearDown() throws Exception {
        super.tearDown();
        deleteRec(dir);
    }

    static void deleteRec(File dir) throws IOException {
        if (dir == null) {
            return;
        }
        File[] arr = dir.listFiles();
        if (arr != null) {
            for (File f : arr) {
                deleteRec(f);
            }
        }
        dir.delete();
    }
    @Test public void testNotLoggedIn() {
        String status  = resource().path("login").queryParam("id", "not-logged-in").
                accept(MediaType.TEXT_PLAIN).get(String.class);
        assertEquals("Nobody is logged in", "", status);
    }


    @Test public void testCreateAGame() throws Exception {
        String logJarda = resource().path("login").
            queryParam("name", "Jarda").
            queryParam("password", "heslo").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);
        String logJirka = resource().path("login").
            queryParam("name", "Jirka").
            queryParam("password", "pesko").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);
        assertNotNull("Logged in ok", logJarda);
        GameId s = resource().path("games").queryParam("white", "Jarda")
                .queryParam("loginID", logJarda)
                .queryParam("black", "Jirka").post(GameId.class);

        for (int i = 0; i < 3; i++) {
            resource().path("games/" + s.getId())
                .queryParam("loginID", logJarda)
                .queryParam("player", "Jarda").queryParam("move", "N").put(GameId.class);
            resource().path("games/" + s.getId())
                .queryParam("loginID", logJirka)
                .queryParam("player", "Jirka").queryParam("move", "S").put(GameId.class);
        }

        resource().path("games/" + s.getId())
            .queryParam("loginID", logJarda)
            .queryParam("player", "Jarda").queryParam("move", "N").put(GameId.class);
        resource().path("games/" + s.getId())
            .queryParam("loginID", logJirka)
            .queryParam("player", "Jirka").queryParam("move", "SS").put(GameId.class);

        
        GenericType<List<GameId>> gType = new GenericType<List<GameId>>() {};
        List<GameId> nothing = resource().path("games/").queryParam("status", "blackWon").accept(MediaType.TEXT_XML).get(gType);
        assertTrue("Nothing has been finished yet: " + nothing, nothing.isEmpty());

        for (int i = 0; i < 3; i++) {
            resource().path("games/" + s.getId())
                .queryParam("loginID", logJarda)
                .queryParam("player", "Jarda").queryParam("move", "N").put(GameId.class);
            resource().path("games/" + s.getId())
                .queryParam("loginID", logJirka)
                .queryParam("player", "Jirka").queryParam("move", "S").put(GameId.class);
        }


        try {
            Game end = resource().path("games/" + s.getId()).accept(MediaType.TEXT_XML).get(Game.class);
            fail("If the game is finished, one cannot get its status without login");
        } catch (UniformInterfaceException ex) {
            // OK
        }
        Game end = resource().path("games/" + s.getId()).queryParam("loginID", logJirka).accept(MediaType.TEXT_XML).get(Game.class);
        assertEquals("BlackWins", GameStatus.blackWon, end.getId().getStatus());

        assertEquals("Jirka wins", "Jirka", end.getCurrentPlayer());

        List<GameId> none = resource().path("games/").queryParam("status", "blackWon").accept(MediaType.TEXT_XML).get(gType);
        assertEquals("No games, for not logged in users: " + none, 0, none.size());

        List<GameId> something = resource().path("games/").queryParam("loginID", logJirka).queryParam("status", "blackWon").accept(MediaType.TEXT_XML).get(gType);
        assertEquals("One game finished: " + something, 1, something.size());
        assertEquals("Id is OK", end.getId().getId(), something.get(0).getId());
    }

    @Test public void testResignAGame() throws Exception {
        String logJarda = resource().path("login").
            queryParam("name", "Jarda").
            queryParam("password", "heslo").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);
        GameId s = resource().path("games").queryParam("white", "Jarda")
                .queryParam("loginID", logJarda)
                .queryParam("black", "Jirka").post(GameId.class);

        assertTrue("In progress", s.getStatus().isInProgress());

        resource().path("games/" + s.getId()).
            queryParam("loginID", logJarda).
            queryParam("player", "Jarda").
            queryParam("move", "RESIGN").put(GameId.class);
        try {
            Game end = resource().path("games/" + s.getId()).accept(MediaType.TEXT_XML).get(Game.class);
            fail("Should not be able to get game when finished");
        } catch (UniformInterfaceException ex) {
            // OK
        }
        Game end = resource().path("games/" + s.getId()).queryParam("loginID", logJarda).accept(MediaType.TEXT_XML).get(Game.class);
        assertEquals("BlackWins", GameStatus.blackWon, end.getId().getStatus());
        assertEquals("Jirka wins", "Jirka", end.getCurrentPlayer());

        assertFalse("is finished", end.getId().getStatus().isInProgress());
    }

    @Test public void testResignBGame() throws Exception {
        String logJarda = resource().path("login").
            queryParam("name", "Jarda").
            queryParam("password", "heslo").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);
        String logJirka = resource().path("login").
            queryParam("name", "Jirka").
            queryParam("password", "pesko").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);

        GameId s = resource().path("games").queryParam("white", "Jarda")
                .queryParam("loginID", logJarda)
                .queryParam("black", "Jirka").post(GameId.class);

        assertTrue("In progress", s.getStatus().isInProgress());

        resource().path("games/" + s.getId()).
            queryParam("loginID", logJarda).
            queryParam("player", "Jarda").
            queryParam("move", "N").put(GameId.class);
        resource().path("games/" + s.getId()).
            queryParam("loginID", logJirka).
            queryParam("player", "Jirka").
            queryParam("move", "RESIGN").put(GameId.class);


        try {
            Game end = resource().path("games/" + s.getId()).accept(MediaType.TEXT_XML).get(Game.class);
            fail("Should not be able to get game when finished");
        } catch (UniformInterfaceException ex) {
            // OK
        }
        Game end = resource().path("games/" + s.getId()).queryParam("loginID", logJarda).accept(MediaType.TEXT_XML).get(Game.class);
        assertEquals("WhiteWins", GameStatus.whiteWon, end.getId().getStatus());
        assertEquals("Jarda wins", "Jarda", end.getCurrentPlayer());

        assertFalse("is finished", end.getId().getStatus().isInProgress());
    }

    @Test public void testResignForeignGame() throws Exception {
        String logJarda = resource().path("login").
            queryParam("name", "Jarda").
            queryParam("password", "heslo").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);
        GameId s = resource().path("games").queryParam("white", "Jarda")
                .queryParam("loginID", logJarda)
                .queryParam("black", "Jirka").post(GameId.class);
        File usersDir = new File(dir, "users");
        usersDir.mkdirs();
        File Master = new File(usersDir, "Master");
        {
            Properties p = new Properties();
            p.setProperty("email", "mas@ter.cz");
            p.setProperty("permission.resign", "true");
            p.store(new FileOutputStream(Master), "");
        }

        assertTrue("In progress", s.getStatus().isInProgress());
        String logMaster = resource().path("login").
                queryParam("name", "Master").
                queryParam("password", "mr").
                accept(MediaType.TEXT_PLAIN).
                put(String.class);

        resource().path("games/" + s.getId()).
            queryParam("loginID", logMaster).
            queryParam("player", "Jarda").
            queryParam("move", "RESIGN").put(GameId.class);
        try {
            Game end = resource().path("games/" + s.getId()).accept(MediaType.TEXT_XML).get(Game.class);
            fail("Should not be able to get game when finished");
        } catch (UniformInterfaceException ex) {
            // OK
        }
        Game end = resource().path("games/" + s.getId()).queryParam("loginID", logJarda).accept(MediaType.TEXT_XML).get(Game.class);
        assertEquals("BlackWins", GameStatus.blackWon, end.getId().getStatus());
        assertEquals("Jirka wins", "Jirka", end.getCurrentPlayer());

        assertFalse("is finished", end.getId().getStatus().isInProgress());
    }

}
